from datoso_seed_whdload.dats import WhdloadDat

rules = [
    {
        'name': 'Whdload Dat',
        '_class': WhdloadDat,
        'seed': 'Whdload',
        'priority': 0,
        'rules': [
            {
                'key': 'url',
                'operator': 'contains',
                'value': 'www._whdload.org',
            },
            {
                'key': 'homepage',
                'operator': 'eq',
                'value': '_whdload',
            },
        ],
    },
]


def get_rules():
    return rules



from datoso_seed_whdload.dats import WhdloadDat

# ruff: noqa: ERA001

actions = {
    '{dat_origin}': [
        {
            'action': 'LoadDatFile',
            '_class': WhdloadDat,
        },
        {
            'action': 'DeleteOld',
        },
        {
            'action': 'Copy',
            'folder': '{dat_destination}',
        },
        {
            'action': 'SaveToDatabase',
        },
    ],
}

def get_actions():
    return actions



"""__init__.py"""
__all__ = ['__version__', '__author__', '__description__']
__version__ = '1.0a0.dev3'
__author__ = 'Lacides Miranda'
__description__ = 'WHDLoad is a collection of Amiga Roms'
__prefix__ = 'whdload'



import re

from datoso.repositories.dat import ClrMameProDatFile


class WhdloadDat(ClrMameProDatFile):
    seed: str = 'Whdload'

    def initial_parse(self) -> list:
        """Parse the dat file."""
        # pylint: disable=R0801
        system, suffix1, suffix2 = self.name.split(' - ')
        self.prefix = 'Computer'
        self.company, self.system = system.split(' ')
        self.suffix = [suffix1, suffix2]
        self.date = ''
        return [self.prefix, self.company, self.system, self.suffix, self.get_date()]

    def get_date(self) -> str:
        """Get the date from the dat file."""
        if self.date:
            return self.date
        if self.version:
            result = re.findall(r'\(.*?\)', str(self.version))
            self.date = result[len(result)-1][1:-1]
        if self.file:
            result = re.findall(r'\(.*?\)', str(self.file))
            self.date = result[len(result)-1][1:-1]
        return self.date



import os
import shutil
import urllib.request
import zipfile
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime
from html.parser import HTMLParser
from pathlib import Path
from urllib.parse import urljoin

from dateutil import tz

from datoso.configuration.folder_helper import Folders
from datoso_seed_whdload import __prefix__

NAME = 'WHDLoad'
WHDLOAD_URL = 'http://armaxweb.free.fr/'

class MyHTMLParser(HTMLParser):
    dats: list | None = None
    rootpath = None

    def handle_starttag(self, tag, attrs):
        if tag == 'a':
            taga = dict(attrs)
            if 'href' in taga:
                href = taga['href']
                if href.endswith('.zip') and href.startswith('dats/'):
                    self.dats.append(urljoin(self.rootpath, href).replace(' ', '%20'))


def download_dats(folder_helper):
    def get_dat_links(name, url):
        # get mame dats
        print(f'Fetching {name} DAT files')
        if not url.startswith(('http:', 'https:')):
            msg = 'URL must start with "http:" or "https:"'
            raise ValueError(msg)
        red = urllib.request.urlopen(url) # noqa: S310
        pleasurehtml = red.read()

        parser = MyHTMLParser()
        parser.dats = []
        parser.rootpath = url
        parser.folder_helper = folder_helper
        parser.feed(str(pleasurehtml))
        return parser.dats

    def download_dat(href):
        filename = Path(href).name.replace('%20', ' ')
        if not href.startswith(('http:', 'https:')):
            msg = 'URL must start with "http:" or "https:"'
            raise ValueError(msg)
        tmp_filename, _ = urllib.request.urlretrieve(href) # noqa: S310
        local_filename = folder_helper.dats / filename
        shutil.move(tmp_filename, local_filename)

    links = get_dat_links(NAME, WHDLOAD_URL)

    print(f'Downloading {NAME} DAT files')
    with ThreadPoolExecutor(max_workers=10) as executor:
        futures = [
            executor.submit(download_dat, href) for href in links
        ]
        for future in futures:
            future.result()

    path = folder_helper.dats
    files = os.listdir(folder_helper.dats)

    for file_zip in files:
        file = path / file_zip
        with zipfile.ZipFile(file, 'r') as zip_ref:
            zip_ref.extractall(path)
        Path.unlink(file)

    print('\nZipping files for backup')
    backup_daily_name = f'whdload-{datetime.now(tz.tzlocal()).strftime("%Y-%m-%d")}.zip'
    with zipfile.ZipFile(folder_helper.backup / backup_daily_name, 'w') as zip_ref:
        for root, _, files in os.walk(folder_helper.dats):
            for file in files:
                zip_ref.write(Path(root) / file, arcname=Path(root).relative_to(folder_helper.dats) / file,
                              compress_type=zipfile.ZIP_DEFLATED, compresslevel=9)


def fetch():
    folder_helper = Folders(seed=__prefix__)
    folder_helper.clean_dats()
    folder_helper.create_all()
    download_dats(folder_helper)
